package com.ruoyi.system.service.impl;

import org.slf4j.LoggerFactory;
import java.util.Iterator;
import com.ruoyi.common.core.utils.bean.BeanValidators;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.domain.SysUserPost;
import java.util.ArrayList;
import com.ruoyi.system.enums.BussLogEnum;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.common.core.utils.uuid.UUID;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.domain.SysPost;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import com.ruoyi.system.api.domain.SysRole;
import java.util.Collection;
import org.springframework.util.CollectionUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.utils.ServletUtils;
import java.util.List;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.common.security.service.TokenService;
import javax.validation.Validator;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.mapper.SysUserPostMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.mapper.SysPostMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SysUserMapper;
import org.slf4j.Logger;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISysUserService;

@Service
public class SysUserServiceImpl implements ISysUserService
{
    private static final Logger log;
    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private SysRoleMapper roleMapper;
    @Autowired
    private SysPostMapper postMapper;
    @Autowired
    private SysUserRoleMapper userRoleMapper;
    @Autowired
    private SysUserPostMapper userPostMapper;
    @Autowired
    private ISysConfigService configService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    protected Validator validator;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ISysBussLogService sysBussLogService;
    
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(final SysUser user) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("dealer")) {
            user.setDealerId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("agent")) {
            user.setAgentId(loginUser.getUserid());
        }
        return this.userMapper.selectUserList(user);
    }
    
    public List<SysUser> selectList(final SysUser user) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("dealer")) {
            user.setDealerId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("agent")) {
            user.setAgentId(loginUser.getUserid());
        }
        return this.userMapper.selectList(user);
    }
    
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectAllocatedList(final SysUser user) {
        return this.userMapper.selectAllocatedList(user);
    }
    
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUnallocatedList(final SysUser user) {
        return this.userMapper.selectUnallocatedList(user);
    }
    
    public SysUser selectUserByUserName(final String userName) {
        return this.userMapper.selectUserByUserName(userName);
    }
    
    public SysUser selectUserById(final Long userId) {
        return this.userMapper.selectUserById(userId);
    }
    
    public String selectUserRoleGroup(final String userName) {
        final List<SysRole> list = this.roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty((Collection)list)) {
            return "";
        }
        return list.stream().map((Function<? super Object, ?>)SysRole::getRoleName).collect((Collector<? super Object, ?, String>)Collectors.joining(","));
    }
    
    public String selectUserPostGroup(final String userName) {
        final List<SysPost> list = this.postMapper.selectPostsByUserName(userName);
        if (CollectionUtils.isEmpty((Collection)list)) {
            return "";
        }
        return list.stream().map((Function<? super Object, ?>)SysPost::getPostName).collect((Collector<? super Object, ?, String>)Collectors.joining(","));
    }
    
    public boolean checkUserNameUnique(final SysUser user) {
        final Long userId = StringUtils.isNull((Object)user.getUserId()) ? -1L : user.getUserId();
        final SysUser info = this.userMapper.checkUserNameUnique(user.getUserName());
        return !StringUtils.isNotNull((Object)info) || info.getUserId() == (long)userId;
    }
    
    public boolean checkPhoneUnique(final SysUser user) {
        final Long userId = StringUtils.isNull((Object)user.getUserId()) ? -1L : user.getUserId();
        final SysUser info = this.userMapper.checkPhoneUnique(user.getPhonenumber());
        return !StringUtils.isNotNull((Object)info) || info.getUserId() == (long)userId;
    }
    
    public boolean checkEmailUnique(final SysUser user) {
        final Long userId = StringUtils.isNull((Object)user.getUserId()) ? -1L : user.getUserId();
        final SysUser info = this.userMapper.checkEmailUnique(user.getEmail());
        return !StringUtils.isNotNull((Object)info) || info.getUserId() == (long)userId;
    }
    
    public void checkUserAllowed(final SysUser user) {
        if (StringUtils.isNotNull((Object)user.getUserId()) && user.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }
    
    public void checkUserDataScope(final Long userId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            final SysUser user = new SysUser();
            user.setUserId(userId);
            final List<SysUser> users = ((SysUserServiceImpl)SpringUtils.getAopProxy((Object)this)).selectUserList(user);
            if (StringUtils.isEmpty((Collection)users)) {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int insertUser(final SysUser user) {
        user.setLicense(UUID.randomUUID().toString().replaceAll("-", ""));
        final int rows = this.userMapper.insertUser(user);
        this.insertUserPost(user);
        this.insertUserRole(user);
        return rows;
    }
    
    public boolean registerUser(final SysUser user) {
        return this.userMapper.insertUser(user) > 0;
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int updateUser(final SysUser user) {
        final Long userId = user.getUserId();
        this.userRoleMapper.deleteUserRoleByUserId(userId);
        this.insertUserRole(user);
        this.userPostMapper.deleteUserPostByUserId(userId);
        this.insertUserPost(user);
        return this.userMapper.updateUser(user);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public void insertUserAuth(final Long userId, final Long[] roleIds) {
        this.userRoleMapper.deleteUserRoleByUserId(userId);
        this.insertUserRole(userId, roleIds);
    }
    
    public int updateUserStatus(final SysUser user) {
        return this.userMapper.updateUser(user);
    }
    
    public int updateUserProfile(final SysUser user) {
        return this.userMapper.updateUser(user);
    }
    
    public boolean updateUserAvatar(final String userName, final String avatar) {
        return this.userMapper.updateUserAvatar(userName, avatar) > 0;
    }
    
    public int resetPwd(final SysUser user) {
        this.sysBussLogService.insertSysBussLog(BussLogEnum.EditPassword.getType(), user.getUserId(), user.getUserName(), "重置密码为" + user.getPassword());
        user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
        return this.userMapper.updateUser(user);
    }
    
    public int resetUserPwd(final String userName, final String password) {
        return this.userMapper.resetUserPwd(userName, password);
    }
    
    public void insertUserRole(final SysUser user) {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }
    
    public void insertUserPost(final SysUser user) {
        final Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty((Object[])posts)) {
            final List<SysUserPost> list = new ArrayList<SysUserPost>();
            for (final Long postId : posts) {
                final SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                list.add(up);
            }
            this.userPostMapper.batchUserPost((List)list);
        }
    }
    
    public void insertUserRole(final Long userId, final Long[] roleIds) {
        if (StringUtils.isNotEmpty((Object[])roleIds)) {
            final List<SysUserRole> list = new ArrayList<SysUserRole>();
            for (final Long roleId : roleIds) {
                final SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            this.userRoleMapper.batchUserRole((List)list);
        }
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int deleteUserById(final Long userId) {
        this.userRoleMapper.deleteUserRoleByUserId(userId);
        this.userPostMapper.deleteUserPostByUserId(userId);
        return this.userMapper.deleteUserById(userId);
    }
    
    @Transactional(rollbackFor = { Exception.class })
    public int deleteUserByIds(final Long[] userIds) {
        for (final Long userId : userIds) {
            this.checkUserAllowed(new SysUser(userId));
            this.checkUserDataScope(userId);
        }
        this.userRoleMapper.deleteUserRole(userIds);
        this.userPostMapper.deleteUserPost(userIds);
        return this.userMapper.deleteUserByIds(userIds);
    }
    
    public String importUser(final List<SysUser> userList, final Boolean isUpdateSupport, final String operName) {
        if (StringUtils.isNull((Object)userList) || userList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        final StringBuilder successMsg = new StringBuilder();
        final StringBuilder failureMsg = new StringBuilder();
        for (final SysUser user : userList) {
            try {
                final SysUser u = this.userMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull((Object)u)) {
                    BeanValidators.validateWithException(this.validator, (Object)user, new Class[0]);
                    this.deptService.checkDeptDataScope(user.getDeptId());
                    final String password = this.configService.selectConfigByKey("sys.user.initPassword");
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    this.userMapper.insertUser(user);
                    ++successNum;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
                }
                else if (isUpdateSupport) {
                    BeanValidators.validateWithException(this.validator, (Object)user, new Class[0]);
                    this.checkUserAllowed(u);
                    this.checkUserDataScope(u.getUserId());
                    this.deptService.checkDeptDataScope(user.getDeptId());
                    user.setUserId(u.getUserId());
                    user.setUpdateBy(operName);
                    this.userMapper.updateUser(user);
                    ++successNum;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
                }
                else {
                    ++failureNum;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            }
            catch (final Exception e) {
                ++failureNum;
                final String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                SysUserServiceImpl.log.error(msg, e);
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        return successMsg.toString();
    }
    
    public void updateUserCloakId(final Long dealerId, final String cloakId) {
        this.userMapper.updateUserCloakId(dealerId, cloakId);
    }
    
    static {
        log = LoggerFactory.getLogger(SysUserServiceImpl.class);
    }
}
